set echo on
column starttime format a28
column endtime format a28
column duration format a26
clear screen
remark  Een tabel met 48 rijen, jaren 2011 t/m 2014, een per maand
remark
create table percentages_per_year (ym,percentage)
as
 select date '2010-12-01' + numtoyminterval(level,'month')
      , case mod(level,3) when 0 then 5 else 10 end
   from dual
connect by level <= 48
/
pause
select * from percentages_per_year
/
pause
clear screen
remark  Een aggregatie MV erop die het totaal per jaar berekend, plus een check constraint
remark
create materialized view log on percentages_per_year
  with sequence, rowid (ym,percentage) including new values
/
create materialized view mv
  refresh fast on commit
as
select to_char(ym,'yyyy') year
     , sum(percentage)    year_percentage
     , count(percentage)  count_percentage
     , count(*)           count_all
  from percentages_per_year
 group by to_char(ym,'yyyy')
/
pause
alter materialized view mv
  add constraint year_percentage_100_ck
  check (year_percentage=100)
/
pause
clear screen
remark  De data in de MV laat zien dat ieder jaar optelt tot 100%
remark
select * from mv
/
pause
clear screen
remark  Start performance-onderzoek door de tabel te analyseren.
remark  Maak daarna een simpele logtabel aan.
remark
exec dbms_stats.gather_table_stats(user,'percentages_per_year')
create table t_log
( year      number(4)
, starttime timestamp(6)
, endtime   timestamp(6)
)
/
pause
clear screen
remark  En twee logprocedures die de start- en eindtijdstip registreren
remark
create procedure log_start (p_year in t_log.year%type)
is
  pragma autonomous_transaction;
begin
  insert into t_log (year,starttime)
  values (p_year,systimestamp)
  ;
  commit;
end log_start;
/
create procedure log_end (p_year in t_log.year%type)
is
  pragma autonomous_transaction;
begin
  update t_log
     set endtime = systimestamp
   where year = p_year
  ;
  commit;
end log_end;
/
pause
clear screen
remark  Een update-procedure die de twaalf rijen voor 1 jaar bijwerkt. Voorzie deze procedure van logging en tracing
remark
create procedure update_percentages (p_year in number)
is
begin
  dbms_application_info.set_module('p',to_char(p_year))
  ;
  dbms_session.session_trace_enable
  ;
  log_start(p_year)
  ;
  update percentages_per_year
     set percentage = case when extract(month from ym) <= 8 then 9 else 7 end
   where extract(year from ym) = p_year
  ;
  commit
  ;
  log_end(p_year)
  ;
end update_percentages;
/
pause
clear screen
remark  Start nu parallel 4 jobs op die ieder voor een jaar de rijen bijwerken.
remark
declare
  l_job_id binary_integer;
begin
  for i in 2011 .. 2014
  loop
    dbms_job.submit(l_job_id,'rwijk.update_percentages('||to_char(i)||');');
  end loop;
end;
/
commit
/
pause
clear screen
remark  Bekijk de logtabel om de doorlooptijden van de 4 jobs duidelijk te krijgen
remark
select year
     , starttime
     , endtime
     , endtime - starttime duration
  from t_log
 order by endtime
/
pause
set echo off
drop procedure update_percentages
/
drop procedure log_end
/
drop procedure log_start
/
drop materialized view mv
/
drop table percentages_per_year purge
/
drop table t_log purge
/
begin
  for r in (select job from dba_jobs)
  loop
    dbms_job.remove(r.job);
  end loop;
  commit;
end;
/
