set echo on
column asg_ean_code format 999999999999999999
column nbr_ean_code format 9999999999999
clear screen
remark  Het brondatamodel
remark
create table netbeheerders
( id       number(6)    not null
, ean_code varchar2(13) not null
, naam     varchar2(10) not null
, constraint nbr_pk primary key (id)
, constraint nbr_uk1 unique (ean_code)
)
/
create table aansluitingen
( id               number(9)    not null
, ean_code         varchar2(18) not null
, nbr_id           number(6)    not null
, overige_kolommen varchar2(200)
, constraint asg_pk primary key (id)
, constraint asg_uk1 unique (ean_code)
, constraint ask_nbr_fk foreign key (nbr_id) references netbeheerders(id)
)
/
create table aansluiting_profielen
( id         number(9)    not null
, asg_id     number(9)    not null
, kolom      varchar2(30) not null
, waarde     varchar2(30) not null
, begindatum date         not null
, einddatum  date
, constraint apl_pk primary key (id)
, constraint apl_uk1 unique (asg_id,kolom,begindatum)
, constraint apl_asg_fk foreign key (asg_id) references aansluitingen(id)
)
/
pause
clear screen
remark  Drie aansluitingen van verschillende netbeheerders met in tijd varierende waarden voor aansluiting_profielen
remark
insert into netbeheerders
select 1, '8700000123456', 'Stedin' from dual union all
select 2, '8700000987654', 'Enexis' from dual union all
select 3, '8700000345678', 'Alliander' from dual
/
insert into aansluitingen
select 11, '870123456789012345', 1, lpad('*',200,'*') from dual union all
select 12, '870123456789012346', 2, lpad('*',200,'*') from dual union all
select 13, '870123456789012347', 3, lpad('*',200,'*') from dual
/
insert into aansluiting_profielen
select 101, 11, 'Kolom1', '1', date '2013-01-01', date '2013-12-31' from dual union all
select 102, 11, 'Kolom1', '2', date '2014-01-01', null from dual union all
select 103, 11, 'Kolom2', '41', date '2013-01-01', date '2013-12-31' from dual union all
select 104, 11, 'Kolom2', '42', date '2014-01-01', null from dual union all
select 105, 12, 'Kolom1', '1', date '2013-01-01', date '2013-12-31' from dual union all
select 106, 12, 'Kolom1', '2', date '2014-01-01', date '2015-12-31' from dual union all
select 107, 12, 'Kolom2', '41', date '2013-01-01', date '2013-12-31' from dual union all
select 108, 12, 'Kolom2', '42', date '2014-01-01', date '2014-12-31' from dual union all
select 109, 13, 'Kolom1', '1', date '2013-01-01', date '2013-12-31' from dual union all
select 110, 13, 'Kolom1', '2', date '2014-01-01', date '2014-12-31' from dual union all
select 111, 13, 'Kolom1', '3', date '2015-01-01', null from dual
/
pause
clear screen
remark  Zo zou de data in het doeldatamodel eruit moeten zien
remark
select asg.id                  asg_id
     , to_number(asg.ean_code) asg_ean_code
     , to_number(nbr.ean_code) nbr_ean_code
     , nbr.naam                nbr_naam
     , apl1.waarde             kolom1
     , apl2.waarde             kolom2
  from                 aansluitingen         asg
       inner join      netbeheerders         nbr  on (asg.nbr_id = nbr.id)
       left outer join aansluiting_profielen apl1 on (asg.id = apl1.asg_id and apl1.kolom = 'Kolom1')
       left outer join aansluiting_profielen apl2 on (asg.id = apl2.asg_id and apl2.kolom = 'Kolom2')
 where sysdate between nvl(apl1.begindatum,sysdate) and nvl(apl1.einddatum,sysdate)
   and sysdate between nvl(apl2.begindatum,sysdate) and nvl(apl2.einddatum,sysdate)
/
pause
clear screen
remark  Poging 1: Zet de zoekvraag gewoon 1-op-1 in een incrementeel verversende MV
remark
create materialized view log on netbeheerders with rowid, sequence
/
create materialized view log on aansluitingen with rowid, sequence
/
create materialized view log on aansluiting_profielen with rowid, sequence
/
pause
create materialized view car_aansluitingen_mv
  refresh fast on commit
as
select asg.rowid               asg_rowid
     , nbr.rowid               nbr_rowid
     , apl1.rowid              apl1_rowid
     , apl2.rowid              apl2_rowid
     , asg.id                  asg_id
     , to_number(asg.ean_code) asg_ean_code
     , to_number(nbr.ean_code) nbr_ean_code
     , nbr.naam                nbr_naam
     , apl1.waarde             kolom1
     , apl2.waarde             kolom2
  from                 aansluitingen         asg
       inner join      netbeheerders         nbr
                       on (asg.nbr_id = nbr.id)
       left outer join aansluiting_profielen apl1
                       on (   asg.id = apl1.asg_id
                          and apl1.kolom = 'Kolom1'
                          and sysdate between apl1.begindatum and nvl(apl1.einddatum,sysdate)
                          )
       left outer join aansluiting_profielen apl2
                       on (   asg.id = apl2.asg_id
                          and apl2.kolom = 'Kolom2'
                          and sysdate between apl2.begindatum and nvl(apl2.einddatum,sysdate)
                          )
/
pause
clear screen
remark  Poging 2: Haal de ANSI-joins eruit en vervang deze door oude joins
remark
create materialized view car_aansluitingen_mv
  refresh fast on commit
as
select asg.rowid               asg_rowid
     , nbr.rowid               nbr_rowid
     , apl1.rowid              apl1_rowid
     , apl2.rowid              apl2_rowid
     , asg.id                  asg_id
     , to_number(asg.ean_code) asg_ean_code
     , to_number(nbr.ean_code) nbr_ean_code
     , nbr.naam                nbr_naam
     , apl1.waarde             kolom1
     , apl2.waarde             kolom2
  from aansluitingen         asg
     , netbeheerders         nbr
     , aansluiting_profielen apl1
     , aansluiting_profielen apl2
 where asg.nbr_id = nbr.id
   and asg.id = apl1.asg_id (+)
   and apl1.kolom (+) = 'Kolom1'
   and asg.id = apl2.asg_id (+)
   and apl2.kolom (+) = 'Kolom2'
   and sysdate between nvl(apl1.begindatum,sysdate) and nvl(apl1.einddatum,sysdate)
   and sysdate between nvl(apl2.begindatum,sysdate) and nvl(apl2.einddatum,sysdate)
/
pause
clear screen
remark  Verwijder de referenties aan sysdate (algemene beperking #1)
remark
create materialized view car_aansluitingen_mv
  refresh fast on commit
as
select asg.rowid               asg_rowid
     , nbr.rowid               nbr_rowid
     , apl1.rowid              apl1_rowid
     , apl2.rowid              apl2_rowid
     , asg.id                  asg_id
     , to_number(asg.ean_code) asg_ean_code
     , to_number(nbr.ean_code) nbr_ean_code
     , nbr.naam                nbr_naam
     , apl1.waarde             kolom1
     , apl2.waarde             kolom2
  from aansluitingen         asg
     , netbeheerders         nbr
     , aansluiting_profielen apl1
     , aansluiting_profielen apl2
 where asg.nbr_id = nbr.id
   and asg.id = apl1.asg_id (+)
   and apl1.kolom (+) = 'Kolom1'
   and asg.id = apl2.asg_id (+)
   and apl2.kolom (+) = 'Kolom2'
/
pause
clear screen
remark  Het werkt, maar levert teveel rijen op
remark
select asg_id
     , asg_ean_code
     , nbr_ean_code
     , nbr_naam
     , kolom1
     , kolom2
  from car_aansluitingen_mv
 order by asg_id
/
pause
clear screen
remark  Verzin een list om toch de systeemdatum te kunnen gebruiken: een applicatiecontext
remark
create context peildatum_ctx using peildatum_pkg accessed globally
/
create package peildatum_pkg
as
  procedure zet_peildatum(p_peildatum in date);
  function peildatum return date;
end peildatum_pkg;
/
create package body peildatum_pkg
as
  procedure zet_peildatum(p_peildatum in date)
  is
  begin
    dbms_session.set_context
    ( 'peildatum_ctx'
    , 'peildatum'
    , to_char(p_peildatum,'yyyy-mm-dd')
    );
  end zet_peildatum
  ;
  function peildatum return date
  is
  begin
    return to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd');
  end peildatum
  ;
end peildatum_pkg;
/
pause
clear screen
remark  Kleine controle dat de peildatum_pkg werkt
remark
exec peildatum_pkg.zet_peildatum(trunc(sysdate))
exec dbms_output.put_line(to_char(peildatum_pkg.peildatum,'yyyy-mm-dd'))
pause
clear screen
remark  Gebruik de peildatum_pkg in de MV
remark
drop materialized view car_aansluitingen_mv
/
create materialized view car_aansluitingen_mv
  refresh fast on commit
as
select asg.rowid               asg_rowid
     , nbr.rowid               nbr_rowid
     , apl1.rowid              apl1_rowid
     , apl2.rowid              apl2_rowid
     , asg.id                  asg_id
     , to_number(asg.ean_code) asg_ean_code
     , to_number(nbr.ean_code) nbr_ean_code
     , nbr.naam                nbr_naam
     , apl1.waarde             kolom1
     , apl2.waarde             kolom2
  from aansluitingen         asg
     , netbeheerders         nbr
     , aansluiting_profielen apl1
     , aansluiting_profielen apl2
 where asg.nbr_id = nbr.id
   and asg.id = apl1.asg_id (+)
   and apl1.kolom (+) = 'Kolom1'
   and asg.id = apl2.asg_id (+)
   and apl2.kolom (+) = 'Kolom2'
   and to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd') 
         between nvl(apl1.begindatum,to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
             and nvl(apl1.einddatum,to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
   and to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd')
         between nvl(apl2.begindatum,to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
             and nvl(apl2.einddatum,to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
/
pause
clear screen
remark  En voila. Het werkt.
remark
select asg_id
     , asg_ean_code
     , nbr_ean_code
     , nbr_naam
     , kolom1
     , kolom2
  from car_aansluitingen_mv
 order by asg_id
/
pause
clear screen
remark  Performance controleren ...
remark
alter session set "_mv_refresh_use_stats" = true
/
create index mvl_apl_i1 on mlog$_aansluiting_profiele (snaptime$$,m_row$$)
/
begin
  dbms_stats.gather_table_stats(user,'mlog$_aansluitingen');
  dbms_stats.gather_table_stats(user,'mlog$_aansluiting_profiele',cascade=>true);
  dbms_stats.gather_table_stats(user,'mlog$_netbeheerders');
end;
/
pause
clear screen
remark  SQL*Trace een update op een aansluiting_profiel
remark
exec dbms_session.set_sql_trace(true)
update aansluiting_profielen
   set waarde = '43'
 where id = 104
/
commit
/
exec dbms_session.set_sql_trace(false)
pause
clear screen
remark  De wijziging is wel doorgekomen.
remark
select asg_id
     , asg_ean_code
     , nbr_ean_code
     , nbr_naam
     , kolom1
     , kolom2
  from car_aansluitingen_mv
 order by asg_id
/
pause
clear screen
remark  Het statement in het trace-bestand, maar dan geformatteerd
remark
explain plan
for
INSERT  /*+ NOAPPEND */ INTO "RWIJK"."CAR_AANSLUITINGEN_MV"
SELECT /*+ NO_MERGE("JV$") */
       "MAS$3".ROWID
      ,"MAS$2".ROWID
      ,"MAS$1".ROWID
      ,"JV$"."RID$"
      ,"MAS$3"."ID"
      ,TO_NUMBER("MAS$3"."EAN_CODE")
      ,TO_NUMBER("MAS$2"."EAN_CODE")
      ,"MAS$2"."NAAM"
      ,"MAS$1"."WAARDE"
      ,"JV$"."WAARDE"
   FROM ( SELECT "MAS$"."ROWID" "RID$" 
              ,  "MAS$".*
            FROM "RWIJK"."AANSLUITING_PROFIELEN" "MAS$"
           WHERE ROWID IN
                 (SELECT /*+ HASH_SJ */
                         CHARTOROWID("MAS$"."M_ROW$$") RID$
                    FROM "RWIJK"."MLOG$_AANSLUITING_PROFIELE" "MAS$"
                   WHERE "MAS$".XID$$ = :1
                 )
        ) "JV$"
      , "AANSLUITING_PROFIELEN" "MAS$1"
      , "NETBEHEERDERS" AS OF SNAPSHOT(:B_SCN)  "MAS$2"
      , "AANSLUITINGEN" AS OF SNAPSHOT(:B_SCN)  "MAS$3"
  WHERE (   "MAS$3"."NBR_ID"="MAS$2"."ID"
        AND "MAS$3"."ID"="MAS$1"."ASG_ID"(+)
        AND "MAS$1"."KOLOM"(+)='Kolom1'
        AND "MAS$3"."ID"="JV$"."ASG_ID"(+)
        AND "JV$"."KOLOM"(+)='Kolom2'
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            >= NVL("MAS$1"."BEGINDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            <= NVL("MAS$1"."EINDDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            >= NVL("JV$"."BEGINDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            <= NVL("JV$"."EINDDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        )
    AND NOT EXISTS
        ( SELECT 1
            FROM "RWIJK"."CAR_AANSLUITINGEN_MV" "SNA2$"
           WHERE ("SNA2$"."ASG_ROWID" = "MAS$3".ROWID)
             AND ("SNA2$"."NBR_ROWID" = "MAS$2".ROWID OR "MAS$2".ROWID IS NULL)
             AND ("SNA2$"."APL1_ROWID" = "MAS$1".ROWID OR "MAS$1".ROWID IS NULL )
             AND "JV$".RID$ IS NULL
        )
    AND NOT EXISTS
        ( SELECT 1
            FROM "AANSLUITING_PROFIELEN" "MAS_INNER$"
               , "AANSLUITINGEN" AS OF SNAPSHOT(:B_SCN)  "MAS_OUTER$"
           WHERE "MAS$3".ROWID = "MAS_OUTER$".ROWID
             AND "JV$".RID$ IS NULL
             AND "MAS_OUTER$"."ID"= "MAS_INNER$"."ASG_ID"
             AND "MAS_INNER$"."KOLOM"='Kolom2'
        )
/
pause
clear screen
remark  Bindingsvariabelen in de AS OF SNAPSHOT clausule werken niet, dus die verwijderen we gemakshalve
remark
explain plan
for
INSERT  /*+ NOAPPEND */ INTO "RWIJK"."CAR_AANSLUITINGEN_MV"
SELECT /*+ NO_MERGE("JV$") */
       "MAS$3".ROWID
      ,"MAS$2".ROWID
      ,"MAS$1".ROWID
      ,"JV$"."RID$"
      ,"MAS$3"."ID"
      ,TO_NUMBER("MAS$3"."EAN_CODE")
      ,TO_NUMBER("MAS$2"."EAN_CODE")
      ,"MAS$2"."NAAM"
      ,"MAS$1"."WAARDE"
      ,"JV$"."WAARDE"
   FROM ( SELECT "MAS$"."ROWID" "RID$" 
              ,  "MAS$".*
            FROM "RWIJK"."AANSLUITING_PROFIELEN" "MAS$"
           WHERE ROWID IN
                 (SELECT /*+ HASH_SJ */
                         CHARTOROWID("MAS$"."M_ROW$$") RID$
                    FROM "RWIJK"."MLOG$_AANSLUITING_PROFIELE" "MAS$"
                   WHERE "MAS$".XID$$ = :1
                 )
        ) "JV$"
      , "AANSLUITING_PROFIELEN" "MAS$1"
      , "NETBEHEERDERS"  "MAS$2"
      , "AANSLUITINGEN"  "MAS$3"
  WHERE (   "MAS$3"."NBR_ID"="MAS$2"."ID"
        AND "MAS$3"."ID"="MAS$1"."ASG_ID"(+)
        AND "MAS$1"."KOLOM"(+)='Kolom1'
        AND "MAS$3"."ID"="JV$"."ASG_ID"(+)
        AND "JV$"."KOLOM"(+)='Kolom2'
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            >= NVL("MAS$1"."BEGINDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            <= NVL("MAS$1"."EINDDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            >= NVL("JV$"."BEGINDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            <= NVL("JV$"."EINDDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        )
    AND NOT EXISTS
        ( SELECT 1
            FROM "RWIJK"."CAR_AANSLUITINGEN_MV" "SNA2$"
           WHERE ("SNA2$"."ASG_ROWID" = "MAS$3".ROWID)
             AND ("SNA2$"."NBR_ROWID" = "MAS$2".ROWID OR "MAS$2".ROWID IS NULL)
             AND ("SNA2$"."APL1_ROWID" = "MAS$1".ROWID OR "MAS$1".ROWID IS NULL )
             AND "JV$".RID$ IS NULL
        )
    AND NOT EXISTS
        ( SELECT 1
            FROM "AANSLUITING_PROFIELEN" "MAS_INNER$"
               , "AANSLUITINGEN"  "MAS_OUTER$"
           WHERE "MAS$3".ROWID = "MAS_OUTER$".ROWID
             AND "JV$".RID$ IS NULL
             AND "MAS_OUTER$"."ID"= "MAS_INNER$"."ASG_ID"
             AND "MAS_INNER$"."KOLOM"='Kolom2'
        )
/
pause
clear screen
remark  Nu zien we wel het plan
remark
select * from table(dbms_xplan.display)
/
pause
clear screen
remark  En nu het plan zonder de HASH_SJ hint
remark
explain plan
for
INSERT  /*+ NOAPPEND */ INTO "RWIJK"."CAR_AANSLUITINGEN_MV"
SELECT /*+ NO_MERGE("JV$") */
       "MAS$3".ROWID
      ,"MAS$2".ROWID
      ,"MAS$1".ROWID
      ,"JV$"."RID$"
      ,"MAS$3"."ID"
      ,TO_NUMBER("MAS$3"."EAN_CODE")
      ,TO_NUMBER("MAS$2"."EAN_CODE")
      ,"MAS$2"."NAAM"
      ,"MAS$1"."WAARDE"
      ,"JV$"."WAARDE"
   FROM ( SELECT "MAS$"."ROWID" "RID$" 
              ,  "MAS$".*
            FROM "RWIJK"."AANSLUITING_PROFIELEN" "MAS$"
           WHERE ROWID IN
                 (SELECT 
                         CHARTOROWID("MAS$"."M_ROW$$") RID$
                    FROM "RWIJK"."MLOG$_AANSLUITING_PROFIELE" "MAS$"
                   WHERE "MAS$".XID$$ = :1
                 )
        ) "JV$"
      , "AANSLUITING_PROFIELEN" "MAS$1"
      , "NETBEHEERDERS" "MAS$2"
      , "AANSLUITINGEN" "MAS$3"
  WHERE (   "MAS$3"."NBR_ID"="MAS$2"."ID"
        AND "MAS$3"."ID"="MAS$1"."ASG_ID"(+)
        AND "MAS$1"."KOLOM"(+)='Kolom1'
        AND "MAS$3"."ID"="JV$"."ASG_ID"(+)
        AND "JV$"."KOLOM"(+)='Kolom2'
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            >= NVL("MAS$1"."BEGINDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            <= NVL("MAS$1"."EINDDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            >= NVL("JV$"."BEGINDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        AND TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd')
            <= NVL("JV$"."EINDDATUM",TO_DATE(SYS_CONTEXT('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
        )
    AND NOT EXISTS
        ( SELECT 1
            FROM "RWIJK"."CAR_AANSLUITINGEN_MV" "SNA2$"
           WHERE ("SNA2$"."ASG_ROWID" = "MAS$3".ROWID)
             AND ("SNA2$"."NBR_ROWID" = "MAS$2".ROWID OR "MAS$2".ROWID IS NULL)
             AND ("SNA2$"."APL1_ROWID" = "MAS$1".ROWID OR "MAS$1".ROWID IS NULL )
             AND "JV$".RID$ IS NULL
        )
    AND NOT EXISTS
        ( SELECT 1
            FROM "AANSLUITING_PROFIELEN" "MAS_INNER$"
               , "AANSLUITINGEN" "MAS_OUTER$"
           WHERE "MAS$3".ROWID = "MAS_OUTER$".ROWID
             AND "JV$".RID$ IS NULL
             AND "MAS_OUTER$"."ID"= "MAS_INNER$"."ASG_ID"
             AND "MAS_INNER$"."KOLOM"='Kolom2'
        )
/
select * from table(dbms_xplan.display)
/
pause
clear screen
remark  Totaal anders: eerst simpelweg de actuele aansluiting_profielen selecteren
remark
drop materialized view car_aansluitingen_mv
/
drop materialized view log on aansluiting_profielen
/
create materialized view log on aansluiting_profielen with primary key
/
pause
create materialized view car_aansluiting_attributen_mv
  refresh fast on commit
as
select apl.id
     , apl.asg_id
     , apl.kolom
     , apl.waarde
  from aansluiting_profielen apl
 where to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd')
         between apl.begindatum
             and nvl(apl.einddatum,to_date(sys_context('peildatum_ctx','peildatum'),'yyyy-mm-dd'))
/
pause
select * from car_aansluiting_attributen_mv
/
pause
clear screen
remark  Daar bovenop een aggregatie MV die de aansluiting-profielen draait (pivot)
remark
create materialized view log on car_aansluiting_attributen_mv
  with rowid,sequence (asg_id,waarde,kolom)
  including new values
/
pause
create materialized view car_agg_aansluiting_attr_mv
  refresh fast on commit
as
select aat.asg_id
     , max(decode(aat.kolom,'Kolom1',to_number(aat.waarde))) kolom1
     , max(decode(aat.kolom,'Kolom2',to_number(aat.waarde))) kolom2
     , count(aat.kolom)  count_kolom
     , count(aat.waarde) count_waarde
     , count(*)          count_all
  from car_aansluiting_attributen_mv aat
 group by aat.asg_id
/
pause
select *
  from car_agg_aansluiting_attr_mv
 order by asg_id
/
pause
clear screen
remark  En daar weer bovenop een join MV die de attributen bij de aansluiting joint.
remark
create materialized view log on car_agg_aansluiting_attr_mv with rowid
/
create materialized view car_aansluitingen_mv
  refresh fast on commit
as
select asg.rowid               asg_rowid
     , nbr.rowid               nbr_rowid
     , aaa.rowid               aaa_rowid
     , asg.id                  asg_id
     , to_number(asg.ean_code) asg_ean_code
     , to_number(nbr.ean_code) nbr_ean_code
     , nbr.naam                nbr_naam
     , aaa.kolom1
     , aaa.kolom2
  from aansluitingen               asg
     , netbeheerders               nbr
     , car_agg_aansluiting_attr_mv aaa
 where asg.nbr_id = nbr.id
   and asg.id = aaa.asg_id
/
pause
select *
  from car_aansluitingen_mv
 order by asg_id
/
pause
clear screen
remark  En kijken of het geheel weer werkt
remark
update aansluiting_profielen
   set waarde = 44
 where id = 104
/
commit
/
pause
select *
  from car_aansluitingen_mv
 order by asg_id
/
pause
set echo off
drop context peildatum_ctx
/
drop package peildatum_pkg
/
drop materialized view car_agg_aansluiting_attr_mv
/
drop materialized view car_aansluiting_attributen_mv
/
drop materialized view car_aansluitingen_mv
/
drop table aansluiting_profielen purge
/
drop table aansluitingen purge
/
drop table netbeheerders purge
/
